library(robfilter) # load package 'robfilter'
setwd(choose.dir()) # choose directory
x.C <- read.table("Data//Situation_C.txt") # load time series for situation A from folder "data"
source("Application_SCARM_and_trend_detection//scarm.detection.R") # load function 'scarm.detection.R'

# prepare data
temp. <- x.C$Temp
vel. <- x.C$Vel  
wid. <- x.C$Width
int. <- x.C$Int  

# wobble data
set.seed(round(temp.)[1])
temp2 <- temp. + rnorm(n=length(temp.),sd=sd(temp.[1:400]/2))
vel2 <- vel. + rnorm(n=length(vel.),sd=sd(vel.[1:400]/2))
wid2 <- wid. + rnorm(n=length(wid.),sd=sd(wid.[1:400]/2))
int2 <- int. + rnorm(n=length(int.),sd=sd(int.[1:400]/2))

# apply R function 'scarm.detection.R'
temp.det <- scarm.detection(temp2, right.width=40, autocorrelations="moderate.positive")
vel.det <- scarm.detection(vel2, right.width=40, autocorrelations="moderate.positive")
wid.det <- scarm.detection(wid2, right.width=40, autocorrelations="moderate.positive")
int.det <- scarm.detection(int2, right.width=40, autocorrelations="moderate.positive")

# compute the statistics of the trend-detection procedure
temp.det.stat <- temp.det$slope.est / sqrt(temp.det$var.slope)
vel.det.stat <- vel.det$slope.est / sqrt(vel.det$var.slope)
wid.det.stat <- wid.det$slope.est / sqrt(wid.det$var.slope)
int.det.stat <- int.det$slope.est / sqrt(int.det$var.slope)

# plot the results of the application; this plot corresponds to Figure 4 in the article
par(mfrow=c(2,2), mgp=c(3,0.75,0), mar=c(4,5,2,1))
plot(temp2, xlab="t", ylab=expression(x[t]^{temp}), type="l",
main="Particle temperature")
abline(v=which(abs(temp.det.stat)>2), col=gray(0.7))
abline(v=which(abs(temp.det.stat)>4), col=gray(0.5))
abline(v=which(temp.det$adapted.width==13)[-1], lty=2)
lines(temp2)
box()
plot(vel2, xlab="t", ylab=expression(x[t]^{vel}), type="l",
main="Particle velocity")
abline(v=which(abs(vel.det.stat)>2), col=gray(0.7))
abline(v=which(abs(vel.det.stat)>4), col=gray(0.5))
abline(v=which(vel.det$adapted.width==13)[-1], lty=2)
lines(vel2)
box()
plot(wid2, xlab="t", ylab=expression(x[t]^{wid}), type="l",
main="Spray jet width")
abline(v=which(abs(wid.det.stat)>2), col=gray(0.7))
abline(v=which(abs(wid.det.stat)>4), col=gray(0.5))
abline(v=which(wid.det$adapted.width==13)[-1], lty=2)
lines(wid2)
box()
plot(int2, xlab="t", ylab=expression(x[t]^{int}), type="l",
main="Spray jet intensity")
abline(v=which(abs(int.det.stat)>2), col=gray(0.7))
abline(v=which(abs(int.det.stat)>4), col=gray(0.5))
abline(v=which(int.det$adapted.width==13)[-1], lty=2)
lines(int2)
box()
