###########################
##########
########## Help routine 
## Matrix for D1hat
helpmatrix <- function(n)
{
helpmatrix <- matrix(rep(0,n^2),ncol=n)
for(i in 1:n)
 {
  for(j in 1:n)
    {
     abstand <- abs(i-j)
     helpmatrix[i,j] <- 1/n * max(0,1-abstand/sqrt(n))
    }
 }
return(helpmatrix)
}



############################
############################
##    MAIN ROUTINE
## Fluctuation test:

testV <- function(X)
{
  n <- length(X)
  mx1 <- mean(X)
  mx2 <- mean(X^2)
  Abl <- c(1,-2*mx1)
  U <- matrix(c(X^2-mx2,X-mx1),ncol=2,byrow=F)
  
  D1dach <- t(U) %*% helpmatrix(n) %*% U
  
  Ddach <- t(Abl) %*% D1dach %*% Abl
  Ddach <- 1/sqrt(Ddach)
  Varn <- mx2 - (mx1)^2
  
  vektor <- rep(0,n)
  Vari <- rep(0,n)
  for(i in 1:n)
  {
    Vari[i] <- mean((X[1:i])^2) - (mean(X[1:i]))^2
    vektor[i] <- i/sqrt(n) * (Vari[i] - Varn)
  }
  
  teststatistic <- max(Ddach * abs(vektor))
  T<-which((Ddach * abs(vektor))==max(Ddach * abs(vektor)))
  return(list(teststatistic,T))
}


#######################
####
####    Data preparation
print("Please select the directory where the data files are stored in!")
setwd(choose.dir())
### Situation A,B,C,D as in manuscript, compare Table 1
situationA<-read.table("Situation_A.txt")
situationB<-read.table("Situation_B.txt")
situationC<-read.table("Situation_C.txt")
situationD<-read.table("Situation_D.txt")

counter<-1
situationAvel<-NULL
situationAtem<-NULL
situationAwid<-NULL
situationAint<-NULL
  
situationBvel<-NULL
situationBtem<-NULL
situationBwid<-NULL
situationBint<-NULL
  
situationDvel<-NULL
situationDtem<-NULL
situationDwid<-NULL
situationDint<-NULL

situationCvel<-NULL
situationCtem<-NULL
situationCwid<-NULL
situationCint<-NULL

#### Filtering of time series and application of the fluctuation test
require(robfilter)
winwid<-seq(11,201,by=10)
for(widthv in winwid)
{
  situationAvel[counter]<-testV(situationA$Vel-unlist(hybrid.filter(situationA$Vel,width=widthv,method="RM")$level))[[1]]
  situationAtem[counter]<-testV(situationA$Temp-unlist(hybrid.filter(situationA$Temp,width=widthv,method="RM")$level))[[1]]
  situationAwid[counter]<-testV(situationA$Width-unlist(hybrid.filter(situationA$Width,width=widthv,method="RM")$level))[[1]]
  situationAint[counter]<-testV(situationA$Int-unlist(hybrid.filter(situationA$Int,width=widthv,method="RM")$level))[[1]]

  situationBvel[counter]<-testV(situationB$Vel-unlist(hybrid.filter(situationB$Vel,width=widthv,method="RM")$level))[[1]]
  situationBtem[counter]<-testV(situationB$Temp-unlist(hybrid.filter(situationB$Temp,width=widthv,method="RM")$level))[[1]]
  situationBwid[counter]<-testV(situationB$Width-unlist(hybrid.filter(situationB$Width,width=widthv,method="RM")$level))[[1]]
  situationBint[counter]<-testV(situationB$Int-unlist(hybrid.filter(situationB$Int,width=widthv,method="RM")$level))[[1]]
  
  situationCvel[counter]<-testV(situationC$Vel-unlist(hybrid.filter(situationC$Vel,width=widthv,method="RM")$level))[[1]]
  situationCtem[counter]<-testV(situationC$Temp-unlist(hybrid.filter(situationC$Temp,width=widthv,method="RM")$level))[[1]]
  situationCwid[counter]<-testV(situationC$Width-unlist(hybrid.filter(situationC$Width,width=widthv,method="RM")$level))[[1]]
  situationCint[counter]<-testV(situationC$Int-unlist(hybrid.filter(situationC$Int,width=widthv,method="RM")$level))[[1]]

  situationDvel[counter]<-testV(situationD$Vel-unlist(hybrid.filter(situationD$Vel,width=widthv,method="RM")$level))[[1]]
  situationDtem[counter]<-testV(situationD$Temp-unlist(hybrid.filter(situationD$Temp,width=widthv,method="RM")$level))[[1]]
  situationDwid[counter]<-testV(situationD$Width-unlist(hybrid.filter(situationD$Width,width=widthv,method="RM")$level))[[1]]
  situationDint[counter]<-testV(situationD$Int-unlist(hybrid.filter(situationD$Int,width=widthv,method="RM")$level))[[1]]

  counter<-counter+1
}

### Figure 1
par(mfrow=c(2,2), mgp=c(3,0.75,0), mar=c(4,5,2,1))
plot(winwid,situationAtem,type="b",main="Situation A: 'Failure powder feeder'",xlab="Window width",ylab="Test statistic value",ylim=c(0,2))
points(winwid,situationAvel,lty=2,pch=2)
points(winwid,situationAwid,lty=2,pch=3)
points(winwid,situationAint,lty=2,pch=4)
lines(winwid,situationAvel,lty=2)
lines(winwid,situationAwid,lty=2)
lines(winwid,situationAint,lty=2)
abline(h=1.358,lty=5)
legend("bottomleft",c("Velocity","Temperature","Flame width","Flame intensity"),lty=2,pch=1:4)

plot(winwid,situationBtem,type="b",main="Situation B: 'Pressure drop of carrier gas'",xlab="Window width",ylab="Test statistic value",ylim=c(0,2))
points(winwid,situationBvel,lty=2,pch=2)
points(winwid,situationBwid,lty=2,pch=3)
points(winwid,situationBint,lty=2,pch=4)
lines(winwid,situationBvel,lty=2)
lines(winwid,situationBwid,lty=2)
lines(winwid,situationBint,lty=2)
abline(h=1.358,lty=5)
legend("bottomleft",c("Velocity","Temperature","Flame width","Flame intensity"),lty=2,pch=1:4)

plot(winwid,situationCtem,type="b",main="Situation C: 'Failure of air cooling'",xlab="Window width",ylab="Test statistic value",ylim=c(0,2))
points(winwid,situationCvel,lty=2,pch=2)
points(winwid,situationCwid,lty=2,pch=3)
points(winwid,situationCint,lty=2,pch=4)
lines(winwid,situationCvel,lty=2)
lines(winwid,situationCwid,lty=2)
lines(winwid,situationCint,lty=2)
abline(h=1.358,lty=5)
legend("bottomleft",c("Velocity","Temperature","Flame width","Flame intensity"),lty=2,pch=1:4)


plot(winwid,situationDtem,type="b",main="Situation D: 'Worn-out acceleration nozzle'",xlab="Window width",ylab="Test statistic value",ylim=c(0,2))
points(winwid,situationDvel,lty=2,pch=2)
points(winwid,situationDwid,lty=2,pch=3)
points(winwid,situationDint,lty=2,pch=4)
lines(winwid,situationDvel,lty=2)
lines(winwid,situationDwid,lty=2)
lines(winwid,situationDint,lty=2)
abline(h=1.358,lty=5)
legend("bottomleft",c("Velocity","Temperature","Flame width","Flame intensity"),lty=2,pch=1:4)



