###########################
##########
########## Help routine 
## Matrix for D1hat
helpmatrix <- function(n)
{
helpmatrix <- matrix(rep(0,n^2),ncol=n)
for(i in 1:n)
 {
  for(j in 1:n)
    {
     abstand <- abs(i-j)
     helpmatrix[i,j] <- 1/n * max(0,1-abstand/sqrt(n))
    }
 }
return(helpmatrix)
}



############################
############################
##    MAIN ROUTINE
## Test for constant variance:

testV <- function(X)
{
n <- length(X)
mx1 <- mean(X)
mx2 <- mean(X^2)
Abl <- c(1,-2*mx1)
U <- matrix(c(X^2-mx2,X-mx1),ncol=2,byrow=F)

D1dach <- t(U) %*% helpmatrix(n) %*% U

Ddach <- t(Abl) %*% D1dach %*% Abl
Ddach <- 1/sqrt(Ddach)
Varn <- mx2 - (mx1)^2

vektor <- rep(0,n)
Vari <- rep(0,n)
for(i in 1:n)
{
Vari[i] <- mean((X[1:i])^2) - (mean(X[1:i]))^2
vektor[i] <- i/sqrt(n) * (Vari[i] - Varn)
}

teststatistic <- max(Ddach * abs(vektor))
T<-which((Ddach * abs(vektor))==max(Ddach * abs(vektor)))
return(list(teststatistic,T))
}

library(robfilter)
winwid<-c(11,31,51)
alphawithout1<-NULL
alphawith1<-matrix(0,nrow=1000,ncol=3)
alphawithout2<-NULL
alphawith2<-matrix(0,nrow=1000,ncol=3)
alphawithout3<-NULL
alphawith3<-matrix(0,nrow=1000,ncol=3)


for(i in 1:1000)
{
  set.seed(100+i)  
  alphawith<-NULL
  white<-rnorm(1000)
  white1<-white+c(rep(0,500),rep(0.5,500))
  white2<-white+c(rep(0,500),rep(2,500))
  white3<-white+c(rep(0,500),rep(5,500))
  alphawithout1[i]<-testV(white1)[[1]]>1.358
  alphawithout2[i]<-testV(white2)[[1]]>1.358
  alphawithout3[i]<-testV(white3)[[1]]>1.358
  zaehl<-1
  cat(paste("Simulation: ",i,"\n",sep=""))
  for(widthv in winwid)
  {
    alphawith1[i,zaehl]<-testV(white1-unlist(hybrid.filter(white1,width=widthv,method="RM")$level))[[1]]>1.358
    alphawith2[i,zaehl]<-testV(white2-unlist(hybrid.filter(white2,width=widthv,method="RM")$level))[[1]]>1.358
    alphawith3[i,zaehl]<-testV(white3-unlist(hybrid.filter(white3,width=widthv,method="RM")$level))[[1]]>1.358
    zaehl<-zaehl+1
    cat(paste("Teil ",zaehl,"\n",sep=""))
  }
  cat("------------------------------------------------")
}
jump1without<-sum(alphawithout1)/1000
jump2without<-sum(alphawithout2)/1000
jump3without<-sum(alphawithout3)/1000
jump1with<-NULL
jump2with<-NULL
jump3with<-NULL
for(i in 1:3)
{
  jump1with[i]<-sum(alphawith1[,i])/1000  
  jump2with[i]<-sum(alphawith2[,i])/1000
  jump3with[i]<-sum(alphawith3[,i])/1000
}

table2<-data.frame(tau=c(0.5,2,5),alpha=c(jump1without,jump2without,jump3without),"n=11"=c(jump1with[1],jump2with[1],jump3with[1]),"n=31"=c(jump1with[2],jump2with[2],jump3with[2]),"n=51"=c(jump1with[3],jump2with[3],jump3with[3]))
print(table2)

