################
## Simulation ##
################
setwd(choose.dir()) # choose directory that contains the simulation programs
library(robfilter) # load package 'robfilter'
source("scarm.detection.for.in.control.R") # this R-program is required to conduct the simulations; it is based on the R function 'scarm.detection.R' 

repeats <- 2 # number of repeats
time.points.SCARM <-  c() # empty vector for ARLs of the SCARM
time.points.TD <- matrix(NA, ncol=3, nrow=repeats) # empty matrix for ARLs of the trend-detection procedure

for(i in 1:repeats){ # the Monte Carlo simulation
    set.seed(i)
    n <- sample(x=10:200, size=1) # starting width is chosen randomly
    time.series <- rnorm(100000) # very long time series of normal noise
    numb.outl <- rbinom(1 , 100000, 0.1) # sample number of outliers in time series
    pos.outl <- sample(1:100000, numb.outl, replace=F) # sample time points of outliers in series     
    time.series[pos.outl] <- rnorm(numb.outl, sd=10) # add outliers from N(0,100) at sampled time points
    result <- scarm.detection.for.in.control(time.series, critvals.TD=c(2,3,4), 
    start.width=n, autocorrelations="no") # apply R function 'scarm.detection.for.in.control.R'
    time.points.SCARM[i] <- result$time.point.SCARM-n+1 # run lengths of the trend-detection procedure
    time.points.TD[i,] <- result$time.points.TD-n+1 # run lengths of the SCARM
    write.table(time.points.SCARM, "RLs.in.control.SCARM.contamination.II.errors.txt", sep=",") # store run lengths in chosen directory
    write.table(time.points.TD, "RLs.in.control.TD.contamination.II.errors.txt", sep=",") # store run lengths in chosen directory
}

####################################
## Analysis of simulation results ##
####################################
RLs.in.control.TD <- read.table("RLs.in.control.TD.contamination.II.errors.txt", sep=",")
RLs.in.control.SCARM <- read.table("RLs.in.control.SCARM.contamination.II.errors.txt", sep=",")[,1]
ARL.in.control.SCARM <- mean(RLs.in.control.SCARM)
ARL.in.control.TD <- apply(RLs.in.control.TD,2,mean, na.rm=T)

RLs.in.control.SCARM
round(ARL.in.control.SCARM)
round(ARL.in.control.TD)
