################
## Simulation ##
################
setwd(choose.dir()) # choose directory that contains the simulation programs
library(robfilter) # load package 'robfilter'
source("scarm.detection.for.in.control.R") # this R-program is required to conduct the simulations; it is based on the R function 'scarm.detection.R' 

repeats <- 2 # number of repeats
time.points.SCARM <-  c() # empty vector for ARLs of the SCARM
time.points.TD <- matrix(NA, ncol=3, nrow=repeats) # empty matrix for ARLs of the trend-detection procedure

for(i in 1:repeats){ # the Monte Carlo simulation
    set.seed(i)
    n <- sample(x=10:200, size=1) # starting width is chosen randomly
    time.series <- rweibull(n=100000, scale=1, shape=2) # very long time series of weibull noise
    mean.x <- gamma(1.5)
    var.x <- gamma(2) - gamma(1.5)^2
    time.series <- (time.series-mean.x)/sqrt(var.x) # standardize the weibull noise
    result <- scarm.detection.for.in.control(time.series, critvals.TD=c(2,3,4), 
    start.width=n, autocorrelations="no") # apply R function 'scarm.detection.for.in.control.R'
    time.points.SCARM[i] <- result$time.point.SCARM-n+1 # run lengths of the trend-detection procedure
    time.points.TD[i,] <- result$time.points.TD-n+1 # run lengths of the SCARM
    write.table(time.points.SCARM, "RLs.in.control.SCARM.weibull.errors.txt", sep=",") # store run lengths in chosen directory
    write.table(time.points.TD, "RLs.in.control.TD.weibull.errors.txt", sep=",") # store run lengths in chosen directory
}

####################################
## Analysis of simulation results ##
####################################
RLs.in.control.TD <- read.table("RLs.in.control.TD.weibull.errors.txt", sep=",")
RLs.in.control.SCARM <- read.table("RLs.in.control.SCARM.weibull.errors.txt", sep=",")[,1]
ARL.in.control.SCARM <- mean(RLs.in.control.SCARM)
ARL.in.control.TD <- apply(RLs.in.control.TD,2,mean, na.rm=T)

RLs.in.control.SCARM
round(ARL.in.control.SCARM)
round(ARL.in.control.TD)
