################
## Simulation ##
################
setwd(choose.dir()) # choose directory that contains the simulation programs
library(robfilter) # load package 'robfilter'
source("scarm.detection.for.out.control.R") # this R-program is required to conduct the simulations; it is based on the R function 'scarm.detection.R' 

repeats <- 1000 # number of repeats
time.points.TD <- matrix(NA, ncol=3, nrow=repeats) # empty matrix for ARLs of the trend-detection procedure
slopes <- c(seq(0.01,0.1,0.01),seq(0.2,1,0.1)) # slopes of the trends

for(b in slopes){ # the Monte Carlo simulation
    for(i in 1:repeats){    
        set.seed(i+(b*repeats))
        n <- sample(x=10:200, size=1) # starting width is chosen randomly
        time.series <- rnorm(100000) # very long time series of normal noise
        time.series[n:100000] <- time.series[n:100000]+b*(1:(100000-n+1)) # add trend of slope b after time point n
        result <- scarm.detection.for.out.control(time.series, critvals.TD=c(2,3,4), 
        start.width=n, autocorrelations="no") # apply R function 'scarm.detection.for.out.control.R'
        time.points.TD[i,] <- result$time.points.TD-n+1 # run lengths of the trend-detection procedure
        write.table(time.points.TD, paste("RLs.out.control.TD.b=",b,".txt", sep=""), sep=",") # store run lengths in chosen directory
    }
}

####################################
## Analysis of simulation results ##
####################################

RLs.out.control.b <- NULL
ARL.out.control.b <- NULL

for(i in 1:19){
    slope <- slopes[i]
    RLs.out.control.b[[i]] <- read.table(paste("RLs.out.control.TD.b=",slope,".txt", sep=""), sep=",")
    ARL.out.control.b[[i]] <- apply(RLs.out.control.b[[i]],2,mean, na.rm=T)
}

ARL.out.control <- as.vector(unlist(ARL.out.control.b))
ARL.out.control.c2 <- ARL.out.control[seq(1,3*19,3)]
ARL.out.control.c3 <- ARL.out.control[seq(2,3*19,3)]
ARL.out.control.c4 <- ARL.out.control[seq(3,3*19,3)]

round(ARL.out.control.c2, digits=1)
round(ARL.out.control.c3, digits=1)
round(ARL.out.control.c4, digits=1)
