data(dfs)
data(var.n)

scarm.detection.for.in.control <- function(time.series, right.width = 30, 
    min.left.width = right.width, 
    min.width = floor(right.width/3), max.width = 200, sign.level = 0.001, 
    bound.noise.sd = 0.01, rtr = TRUE, autocorrelations = "automatic", 
    start.width=10, critvals.TD=c(2,3,4)) 
{
    r <- round(right.width)
    ell.min <- round(min.left.width)
    n.min <- round(min.width)
    n.max <- round(max.width)
    length.series <- length(time.series)
   
    get.B.t <- function(y) {
        n <- length(y)
        j <- 1:n
        B.t <- matrix(NA, n, n)
        for (i in j) {
            B.t[j[j < i], i] <- B.t[i, j[j < i]] <- (y[i] - y[j[j < 
                i]])/(i - j[j < i])
        }
        return(B.t)
    }
    get.beta.RM <- function(B.t) {
        beta.i <- apply(B.t, 1, median, na.rm = T)
        beta.RM <- median(beta.i, na.rm = T)
        return(beta.RM)
    }
    get.mu.RM <- function(y, beta.RM) {
        n <- length(y)
        j <- 1:n
        mu.RM <- median(y - (beta.RM * (j - n)), na.rm = T)
        return(mu.RM)
    }
    get.B1 <- function(y, ell) {
        n <- length(y)
        r <- n - ell
        I.left <- 1:ell
        I.right <- (ell + 1):n
        B1 <- matrix(NA, ell, r)
        for (i in I.left) {
            B1[i, I.right - ell] <- (y[i] - y[I.right])/(i - 
                I.right)
        }
        return(B1)
    }
    if (autocorrelations == "high.negative") {
        c.q.sim <- const.Q[, 1]
        c.q.model <- function(n) {
            c.q.sim[300]
        }
    }
    if (autocorrelations == "moderate.negative") {
        c.q.sim <- const.Q[, 2]
        c.q.model <- function(n) {
            c.q.sim[300]
        }
    }
    if (autocorrelations == "small.negative") {
        c.q.sim <- const.Q[, 3]
        c.q.model <- function(n) {
            c.q.sim[300]
        }
    }
    if (autocorrelations == "no") {
        c.q.sim <- const.Q[, 4]
        c.q.model <- function(n) {
            1.21 * (n/(n + 0.44))
        }
    }
    if (autocorrelations == "small.positive") {
        c.q.sim <- const.Q[, 5]
        c.q.model <- function(n) {
            c.q.sim[300]
        }
    }
    if (autocorrelations == "moderate.positive") {
        c.q.sim <- const.Q[, 6]
        c.q.model <- function(n) {
            c.q.sim[300]
        }
    }
    if (autocorrelations == "high.positive") {
        c.q.sim <- const.Q[, 7]
        c.q.model <- function(n) {
            c.q.sim[300]
        }
    }
    get.Q.adj <- function(y, phi.est) {
        n <- length(y)
        m <- length(na.omit(y))
        if (m < 3) 
            stop("data vector must contain at least 3 observations")
        x <- which(!is.na(y))
        y <- y[x]
        if (autocorrelations == "automatic") {
            autocor.type <- which.min(abs(phi.est - c(-0.9, -0.6, 
                -0.3, 0, 0.3, 0.6, 0.9)))
            c.q.sim <- const.Q[, autocor.type]
            c.q.model <- function(n) {
                c.q.sim[300]
            }
        }
        if (m > 4) {
            if (m <= 150) {
                c.q <- c.q.sim[m]
            }
            else {
                c.q <- c.q.model(m)
            }
        }
        else {
            c.q <- 1
        }
        h <- numeric(m - 2)
        for (j in 1:(m - 2)) {
            h[j] <- abs(y[j + 1] - y[j] - (x[j + 1] - x[j]) * 
                ((y[j + 2] - y[j])/(x[j + 2] - x[j])))
        }
        h <- sort(h)
        out <- c.q * sort(h)[max(1, floor(0.5 * (m - 2)))]
        return(max(out, bound.noise.sd))
    }
    get.critval <- function(ell, r, sign.level) {
        if (ell <= 100) {
            ell <- ell - ell%%5
            r <- r - r%%5
            degree.of.freedom <- dfs[ell/5, r/5]
            critval <- qt(p = 1 - (sign.level/2), df = degree.of.freedom)
        }
        else {
            critval <- qnorm(p = 1 - (sign.level/2))
        }
        return(critval)
    }
    if (autocorrelations == "high.negative") {
        emp.var <- var.n[, 1]
        v.model <- function(n) {
            out <- 0.0003514981 + (-0.0629949278/n) + (1.6446684583/(n^2)) + 
                (-3.1859306183/(n^3))
            return(out)
        }
    }
    if (autocorrelations == "moderate.negative") {
        emp.var <- var.n[, 2]
        v.model <- function(n) {
            out <- 0.0002560433 + (-0.043480645/n) + (0.922413038/(n^2)) + 
                (5.4903584072/(n^3))
            return(out)
        }
    }
    if (autocorrelations == "small.negative") {
        emp.var <- var.n[, 3]
        v.model <- function(n) {
            out <- 0.0002385982 + (-0.0395730416/n) + (0.7798496595/(n^2)) + 
                (9.7595002645/(n^3))
            return(out)
        }
    }
    if (autocorrelations == "no") {
        emp.var <- var.n[, 4]
        v.model <- function(n) {
            out <- 4.77e-07 + (17.71 * (1/n^3))
            return(out)
        }
    }
    if (autocorrelations == "small.positive") {
        emp.var <- var.n[, 5]
        v.model <- function(n) {
            out <- 0.000473727 + (-0.086723762/n) + (2.442400804/(n^2)) + 
                (10.578915504/(n^3))
            return(out)
        }
    }
    if (autocorrelations == "moderate.positive") {
        emp.var <- var.n[, 6]
        v.model <- function(n) {
            out <- 0.0007355669 + (-0.1485876684/n) + (5.5086877566/(n^2)) + 
                (-5.3249305456/(n^3))
            return(out)
        }
    }
    if (autocorrelations == "high.positive") {
        emp.var <- var.n[, 7]
        v.model <- function(n) {
            out <- -0.0003318919 + (0.0476670483/n) + (2.2422001348/(n^2)) + 
                (-5.8731008014/(n^3))
            return(out)
        }
    }
    scarm.test.statistic <- function(y, B, ell, r) {
        n <- ell + r
        left.sample <- y[1:ell]
        right.sample <- y[(ell + 1):n]
        B.t.left <- B[1:ell, 1:ell]
        B.t.right <- B[(ell + 1):n, (ell + 1):n]
        beta.left <- get.beta.RM(B.t.left)
        beta.right <- get.beta.RM(B.t.right)
        d.t <- beta.left - beta.right
        if (autocorrelations == "automatic") {
            autocor.type <- which.min(abs(phi.est - c(-0.9, -0.6, 
                -0.3, 0, 0.3, 0.6, 0.9)))
            emp.var <- var.n[, autocor.type]
            v.model <- function(n) {
                emp.var[300]
            }
        }
        if (ell <= 300) {
            v.left <- emp.var[ell]
        }
        else {
            v.left <- v.model(ell)
        }
        if (r <= 300) {
            v.right <- emp.var[r]
        }
        else {
            v.right <- v.model(r)
        }
        var.t <- noise.sd.est^2 * (v.left + v.right)
        test.statistic <- d.t/sqrt(var.t)
        return(list(test.statistic = test.statistic, beta.left = beta.left, 
            beta.right = beta.right))
    }
    
    ###############################################################################
    ###############################################################################
    ###############################################################################
    
    var.slope.comp <- function(noise.sd.est, n){
        if(autocorrelations == "automatic") {
            autocor.type <- which.min(abs(phi.est - c(-0.9, -0.6, -0.3, 0, 0.3, 0.6, 0.9)))
            emp.var <- var.n[, autocor.type]
            v.model <- function(n) {
                emp.var[300]
            }
        }
        if(n <= 300){
            v.const <- emp.var[n]
        } else {
            v.const <- v.model(n)
        }
        return(noise.sd.est^2 * v.const)
    }
    
    ###############################################################################
    ###############################################################################
    ###############################################################################

    signal.est <- slope.est <- slope.diff <- adapted.width <- scarm.statistic <- critvals <- noise.sd <- acf.lag.one <- var.slope <- TD.statistic <- rep(NA, length.series)
    break.check <- rep(0,length(critvals.TD)+1)
    time.point.SCARM <- NA
    time.points.TD <- rep(NA, length(critvals.TD))
    n <- start.width
    I.win <- 1:n
    y <- time.series[I.win]
    B <- get.B.t(y)
    for (i in start.width:length.series) {
        phi.est.sample.data <- time.series[(max(i - n.max + 1, 1)):i]
        phi.est.sample.signal <- signal.est[(max(i - n.max + 1, 1)):i]
        phi.est.sample <- na.omit(phi.est.sample.data - phi.est.sample.signal)
        if(length(phi.est.sample) >= n.min) {
            phi.est <- acf(phi.est.sample, plot = F)$acf[2]
            if (is.na(phi.est)) {
                phi.est <- 0
            }
        } else {
            phi.est <- 0
        }
            if (n >= ell.min + r) {
                ell <- n - r
                  noise.sd.est <- get.Q.adj(y, phi.est = phi.est)
                  critval <- get.critval(ell = ell, r = r, sign.level = sign.level)
                  scarm.test <- scarm.test.statistic(y = y, B = B, ell = ell, r = r)
                  scarm.statistic[i] <- scarm.test$test.statistic
                  slope.diff[i] <- scarm.test$beta.left - scarm.test$beta.right
                  critvals[i] <- critval
                  if (abs(scarm.test$test.statistic) > critval) {
                    # SCARM-Test rejects!
                    if(break.check[1]==0){
                        break.check[1] <- 1 
                        time.point.SCARM <- i
                    }
                    B <- B[(n - n.min + 1):n, (n - n.min + 1):n]
                    beta.est <- get.beta.RM(B)
                    mu.est <- get.mu.RM(beta.RM = beta.est, y = y[(n - 
                      n.min + 1):n])
                    n <- n.min
                    if (rtr == TRUE) {
                      rtr.sample <- time.series[(i - n.min + 
                        1):i]
                      min.rtr.sample <- min(rtr.sample, na.rm = T)
                      max.rtr.sample <- max(rtr.sample, na.rm = T)
                      mu.est <- max(min.rtr.sample, min(mu.est, 
                        max.rtr.sample))
                    }
                    if (i != length.series) {
                      b.new <- (time.series[i + 1] - time.series[i + 
                        c((-n + 1):0)])/(i + 1 - (i + c((-n + 
                        1):0)))
                      B <- cbind(rbind(B, b.new), c(b.new, NA))
                      adapted.width[i] <- n
                      var.slope.est <- var.slope.comp(noise.sd.est, n)
                      y <- time.series[(i - n + 1):(i + 1)]
                      n <- n + 1
                    }
                  } else {
                    beta.est <- get.beta.RM(B)
                    mu.est <- get.mu.RM(beta.RM = beta.est, y = y)
                    adapted.width[i] <- n
                    var.slope.est <- var.slope.comp(noise.sd.est, n)
                    if (i != length.series) {
                      if (n == n.max) {
                        b.new <- (time.series[i + 1] - time.series[i + 
                          c((-n + 2):0)])/(i + 1 - (i + c((-n + 
                          2):0)))
                        B <- cbind(rbind(B[-1, -1], b.new), c(b.new, 
                          NA))
                        y <- time.series[(i - n + 2):(i + 1)]
                      }
                      else {
                        b.new <- (time.series[i + 1] - time.series[i + 
                          c((-n + 1):0)])/(i + 1 - (i + c((-n + 
                          1):0)))
                        B <- cbind(rbind(B, b.new), c(b.new, 
                          NA))
                        y <- time.series[(i - n + 1):(i + 1)]
                        n <- n + 1
                      }
                    }
                    if (rtr == TRUE) {
                      rtr.sample <- time.series[(i - n.min + 
                        1):i]
                      min.rtr.sample <- min(rtr.sample, na.rm = T)
                      max.rtr.sample <- max(rtr.sample, na.rm = T)
                      mu.est <- max(min.rtr.sample, min(mu.est, 
                        max.rtr.sample))
                    }
                  }
            } else {
                beta.est <- get.beta.RM(B)
                mu.est <- get.mu.RM(y = y, beta.RM = beta.est)
                noise.sd.est <- get.Q.adj(y, phi.est = phi.est)
                adapted.width[i] <- n
                var.slope.est <- var.slope.comp(noise.sd.est, n)
                if (rtr == TRUE) {
                  rtr.sample <- time.series[(i - n.min + 1):i]
                  min.rtr.sample <- min(rtr.sample, na.rm = T)
                  max.rtr.sample <- max(rtr.sample, na.rm = T)
                  mu.est <- max(min.rtr.sample, min(mu.est, max.rtr.sample))
                }
                if (i != length.series) {
                  if (n == n.max) {
                    b.new <- (time.series[i + 1] - time.series[i + 
                      c((-n + 2):0)])/(i + 1 - (i + c((-n + 2):0)))
                    B <- cbind(rbind(B[-1, -1], b.new), c(b.new, 
                      NA))
                    y <- time.series[(i - n + 2):(i + 1)]
                  }
                  else {
                    b.new <- (time.series[i + 1] - time.series[i + 
                      c((-n + 1):0)])/(i + 1 - (i + c((-n + 1):0)))
                    B <- cbind(rbind(B, b.new), c(b.new, NA))
                    y <- time.series[(i - n + 1):(i + 1)]
                    n <- n + 1
                  }
                }
            }
        TD.statistic[i] <- beta.est/sqrt(var.slope.est)
        
        for(j in 1:length(critvals.TD)){
            if(abs(TD.statistic[i]) > critvals.TD[j]){
            # TD-procedure finds trend!
                if(break.check[j+1]==0){
                    time.points.TD[j] <- i
                    break.check[j+1] <- 1
                }                
            }
        }

        signal.est[i] <- mu.est
        slope.est[i] <- beta.est
        noise.sd[i] <- noise.sd.est
        acf.lag.one[i] <- phi.est
        var.slope[i] <- var.slope.est
        if(all(break.check==1)){
            break
        }
    }
    result <- list(signal.est = signal.est, slope.est = slope.est, var.slope = var.slope,
        adapted.width = adapted.width, scarm.statistic = scarm.statistic, 
        critvals = critvals, noise.sd = noise.sd, slope.diff = slope.diff, 
        acf.lag.one = acf.lag.one, time.series = time.series, 
        right.width = r, min.left.width = ell.min, min.width = n.min, 
        max.width = n.max, sign.level = sign.level, bound.noise.sd = bound.noise.sd, 
        rtr = rtr, autocorrelations = autocorrelations,
        TD.statistic = TD.statistic,
        time.point.SCARM = time.point.SCARM, time.points.TD = time.points.TD)
    return(structure(result, class = "scarm.detection"))
}
